@@ -1,53 +1,44 @@
 import functools
+from .engin import Engin
 from . import sql_support
-from .constant import Engin
-from .log_support import logger, insert_log, do_sql_log
+from .constant import MYSQL_CONNECTOR
+from .init_import import import_driver
+from .log_support import logger, insert_log, save_log, get_log
 from .support import DBCtx, ConnectionCtx, Dict, MultiColumnsError, TransactionCtx, try_commit, DBError, DB_LOCK
 
 _DB_CTX = None
+_SHOW_SQL = False
 
 
-def init_engin(connect=None, user='root', password='', database='test', host='127.0.0.1', port=3306, show_sql=False, engin=Engin.MySQL, **kwargs):
-    sql_support.set_engin(engin)
-    sql_support.set_show_sql(show_sql)
-    if 'debug' in kwargs:
-        if kwargs.pop('debug'):
-            from logging import DEBUG
-            logger.setLevel(DEBUG)
+def init_db(driver='', pool_size=0, show_sql=False, debug=False, **kwargs):
+    global _DB_CTX, _SHOW_SQL
+    if debug:
+        from logging import DEBUG
+        logger.setLevel(DEBUG)
 
-    if connect:
-        import types
-        assert isinstance(connect, types.FunctionType), '`connect` must be a function, you can use like `connect=lambda: connect(**kwargs)`.'
-        _init_engin(connect)
+    prepared = MYSQL_CONNECTOR == driver
+    engin, driver, creator = import_driver(driver)
+    if pool_size <= 0:
+        connect = lambda: creator.connect(**kwargs)
+    elif prepared:
+        # mysql.connector 用自带连接池
+        kwargs['pool_size'] = pool_size
+        connect = lambda: creator.connect(**kwargs)
     else:
-        kwargs['user'] = user
-        kwargs['password'] = password
-        kwargs['database'] = database
-        kwargs['host'] = host
-        kwargs['port'] = port
-        _init_engin(lambda: connect(**kwargs))
+        from .pooling import pooled_connect
+        connect = pooled_connect(creator, pool_size, **kwargs)
 
-
-def _import(engin: Engin):
-    if engin == Engin.MySQL:
-        try:
-            from mysql.connector import connect
-        except ImportError:
-            from pymysql import connect
-    elif engin == Engin.PostgreSQL:
-        from psycopg2 import connect
-    else:
-        raise DBError('Engin is invalid: %s', engin)
-
-
-def _init_engin(connect):
-    global _DB_CTX
     with DB_LOCK:
         if _DB_CTX is not None:
             raise DBError('DB is already initialized.')
+        _SHOW_SQL = show_sql
+        _DB_CTX = DBCtx(connect=connect, prepared=prepared)
 
-        _DB_CTX = DBCtx(connect=connect)
-        logger.info('Init db engine <%s> ok without connection pool.' % hex(id(_DB_CTX)))
+    Engin.init(engin)
+    if pool_size > 0:
+        logger.info("Inited db engin <%s> of %s with driver: '%s' and pool size: %d." % (hex(id(_DB_CTX)), engin, driver, pool_size))
+    else:
+        logger.info("Inited db engin <%s> of %s with driver: '%s'." % (hex(id(_DB_CTX)), engin, driver))
 
 
 def connection():
@@ -70,12 +61,10 @@ def with_connection(func):
     """
 
     global _DB_CTX
-
     @functools.wraps(func)
     def _wrapper(*args, **kw):
         with ConnectionCtx(_DB_CTX):
             return func(*args, **kw)
-
     return _wrapper
 
 
@@ -102,12 +91,10 @@ def with_transaction(func):
          r = update('update person set passwd=? where id=?', name.upper(), id)
     """
     global _DB_CTX
-
     @functools.wraps(func)
     def _wrapper(*args, **kw):
         with TransactionCtx(_DB_CTX):
             return func(*args, **kw)
-
     return _wrapper
 
 
@@ -119,7 +106,7 @@ def execute(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('execute', sql.strip(), *args)
+    sql = _before_execute('execute', sql.strip(), *args)
     try:
         cursor = _DB_CTX.connection.cursor()
         cursor.execute(sql, args)
@@ -143,35 +130,36 @@ def insert(table: str, **kwargs):
     return execute(sql, *args)
 
 
-def save(pk_sql: str, table: str, **kwargs):
+def save(select_key: str, table: str, **kwargs):
     """
     Insert data into table, return primary key.
-    :param pk_sql: sql for getting primary key
+    :param select_key: sql for select primary key
     :param table: table
     :param kwargs:
     :return: Primary key
     """
-    insert_log('save', table, **kwargs)
+    save_log('save', select_key, table, **kwargs)
     sql, args = sql_support.insert_sql_args(table.strip(), **kwargs)
-    return save_sql(pk_sql, sql, *args)
+    return save_sql(select_key, sql, *args)
 
 
 @with_connection
-def save_sql(pk_sql: str, sql: str, *args):
+def save_sql(select_key: str, sql: str, *args):
     """
     Insert data into table, return primary key.
-    :param pk_sql: sql for getting primary key
+    :param select_key: sql for select primary key
     :param sql: table
     :param args:
     :return: Primary key
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('save', sql, *args)
+    logger.debug("Exec func 'sqlexec.%s', 'select_key': %s \n\t sql: %s \n\t args: %s" % ('save_sql', select_key, sql, args))
+    sql = _before_execute('save_sql', sql, *args)
     try:
         cursor = _DB_CTX.connection.cursor()
         cursor.execute(sql, args)
-        cursor.execute(pk_sql)
+        cursor.execute(select_key)
         result = cursor.fetchone()
         try_commit(_DB_CTX)
         return result[0]
@@ -180,6 +168,19 @@ def save_sql(pk_sql: str, sql: str, *args):
             cursor.close()
 
 
+def batch_insert(table: str, *args):
+    """
+    Batch insert
+    :param table: table name
+    :param args: All number must have same key. [{'name': '张三', 'age': 20}, {'name': '李四', 'age': 28}]
+    :return: Effect row count
+    """
+    logger.debug("Exec func 'sqlexec.%s' \n\t Table: '%s', args: %s" % ('batch_insert', table, args))
+    assert len(args) > 0, 'args should not be empty.'
+    sql, args = sql_support.batch_insert_sql_args(table, *args)
+    return batch_execute(sql, *args)
+
+
 @with_connection
 def batch_execute(sql: str, *args):
     """
@@ -190,7 +191,9 @@ def batch_execute(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('batch_execute', sql.strip(), *args)
+    if isinstance(args[0], dict):
+        sql, args = sql_support.batch_named_sql_args(sql, *args)
+    sql = _before_execute('batch_execute', sql.strip(), *args)
     args = sql_support.get_batch_args(*args)
     try:
         cursor = _DB_CTX.cursor()
@@ -209,7 +212,7 @@ def get(sql: str, *args):
     MultiColumnsError: Expect only one column.
     sql: SELECT count(1) FROM person WHERE name=? and age=? limit 1  -->  args: ('张三', 20)
     """
-    do_sql_log('get', sql, *args)
+    get_log('get', sql, *args)
     result = select_one(sql, *args)
     if result:
         if len(result) == 1:
@@ -228,7 +231,7 @@ def select(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('select', sql.strip(), *args)
+    sql = _before_execute('select', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -246,7 +249,7 @@ def select_one(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('select_one', sql.strip(), *args)
+    sql = _before_execute('select_one', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -256,6 +259,12 @@ def select_one(sql: str, *args):
             cursor.close()
 
 
+# def select_page(sql: str, page_num, page_size, *args):
+#     page_log('select_page', sql.strip(), page_num, page_size, args)
+#     sql, args = sql_support.page_sql_args(sql, page_num, page_size, *args)
+#     return select(sql, *args)
+
+
 @with_connection
 def query(sql: str, *args):
     """
@@ -264,7 +273,7 @@ def query(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('query', sql.strip(), *args)
+    sql = _before_execute('query', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -287,7 +296,7 @@ def query_one(sql: str, *args):
     """
     global _DB_CTX
     cursor = None
-    sql = sql_support.before_execute('query_one', sql.strip(), *args)
+    sql = _before_execute('query_one', sql.strip(), *args)
     try:
         cursor = _DB_CTX.cursor()
         cursor.execute(sql, args)
@@ -301,9 +310,21 @@ def query_one(sql: str, *args):
             cursor.close()
 
 
+# def query_page(sql: str, page_num, page_size, *args):
+#     page_log('query_page', sql.strip(), page_num, page_size, args)
+#     sql, args = sql_support.page_sql_args(sql, page_num, page_size, *args)
+#     return query(sql, *args)
+
+
 def get_connection():
     global _DB_CTX
-    if _DB_CTX.is_not_init():
-        _DB_CTX.init()
+    _DB_CTX.try_init()
     return _DB_CTX.connection
 
+
+def _before_execute(function: str, sql: str, *args):
+    if _SHOW_SQL:
+        logger.info("Exec func 'sqlexec.%s' \n\tSQL: %s \n\tARGS: %s" % (function, sql, args))
+    if '%' in sql and 'like' in sql.lower():
+        sql = sql.replace('%', '%%').replace('%%%%', '%%')
+    return sql.replace('?', '%s')
